#ifndef _RTTSS_H
#define _RTTSS_H

#include "rttypes.h"
#include "rtrunstate.h"

/*
 * return codes for TSS functions
 */
#define TSS_NOOP	 1
#define TSS_OK		 0
#define TSS_FAIL	-1
#define TSS_EOF		-2
#define TSS_NOSERVER	-3
#define TSS_INVALID	-4
#define TSS_SYSERROR	-5
#define TSS_NOTFOUND	-6
#define TSS_ABORT	-7
#define TSS_INIT	-8

/*
 * context keys
 */
enum ContextKey {
        CTXT_workingDir,
        CTXT_datapoolDir,
        CTXT_timeZero,
        CTXT_todZero,
        CTXT_logDir,
        CTXT_logFile,
        CTXT_logData,
        CTXT_testScript,
        CTXT_style,
        CTXT_sourceUID,
	CTXT_testScriptSource,
        CTXT_END
};
typedef enum ContextKey ContextKey;


/*
 * values for CTXT_style
 */
#define TSS_COMPUTER_BASED	((void *)0)
#define TSS_USER_BASED		((void *)1)

/*
 * RunState
 */
typedef s32 RunState;

/*
 * datapool open flags
 */
#define TSS_DP_RANDOM		0x0001
#define TSS_DP_SEQUENTIAL	(TSS_DP_RANDOM << 4)
#define TSS_DP_SHUFFLE		(TSS_DP_RANDOM << 8)
#define TSS_DP_RANDOM_MASK	(TSS_DP_RANDOM | TSS_DP_SEQUENTIAL | TSS_DP_SHUFFLE)

#define TSS_DP_WRAP		0x0002
#define TSS_DP_NOWRAP		(TSS_DP_WRAP << 4)
#define TSS_DP_WRAP_MASK	(TSS_DP_WRAP | TSS_DP_NOWRAP)

#define TSS_DP_SHARED		0x0004
#define TSS_DP_PRIVATE		(TSS_DP_SHARED << 4)
#define TSS_DP_SHARED_MASK	(TSS_DP_SHARED | TSS_DP_PRIVATE)

#define TSS_DP_PERSIST		0x0008

#define TSS_DP_NO_OPEN		0x1000
#define TSS_DP_OPEN		0x2000
#define TSS_DP_SHMEM		0x4000
#define TSS_DP_REWIND		0x8000


/*
 * command_status flags
 */
#define TSS_CMD_STAT_FAIL	0x00
#define TSS_CMD_STAT_PASS	0x01
#define TSS_CMD_STAT_WARN	0x02
#define TSS_CMD_STAT_INFO	0x04


/*
 * log event result codes
 */
#define TSS_LOG_RESULT_NONE	0
#define TSS_LOG_RESULT_PASS	1
#define TSS_LOG_RESULT_FAIL	2
#define TSS_LOG_RESULT_WARN	3
#define TSS_LOG_RESULT_STOPPED 4
#define TSS_LOG_RESULT_INFO	5
#define TSS_LOG_RESULT_COMPLETED 6
#define TSS_LOG_RESULT_UNEVALUATED 7


/*
 * LogEvent_control values
 *
 * both the correct result bit and the type bit must be present for an
 * event to be logged.
 */

/* TMS_LOG_EVENT_RESULT filters */
#define	TSS_LOG_EVENT_CONTROL_NONE		0x00000001
#define	TSS_LOG_EVENT_CONTROL_PASS		0x00000002
#define	TSS_LOG_EVENT_CONTROL_FAIL		0x00000004
#define	TSS_LOG_EVENT_CONTROL_WARNING		0x00000008
#define	TSS_LOG_EVENT_CONTROL_STOPPED		0x00000010
#define	TSS_LOG_EVENT_CONTROL_INFORMATIONAL	0x00000020
#define	TSS_LOG_EVENT_CONTROL_COMPLETED		0x00000040
#define	TSS_LOG_EVENT_CONTROL_UNEVALUATED	0x00000080

#define	TSS_LOG_EVENT_CONTROL_ANYRESULT		0x000000FF

/* TMS_LOG_EVENT_TYPE (group) filters */
#define	TSS_LOG_EVENT_CONTROL_TIMERS		0x00010000
#define	TSS_LOG_EVENT_CONTROL_COMMANDS		0x00020000
#define	TSS_LOG_EVENT_CONTROL_ENVIRON		0x00040000
#define	TSS_LOG_EVENT_CONTROL_STUBS		0x00080000
#define	TSS_LOG_EVENT_CONTROL_TSSERROR		0x00100000
#define	TSS_LOG_EVENT_CONTROL_TSSPROXYERROR	0x00200000

#define	LOG_EVENT_MINIMAL	0x00000000

#define	LOG_EVENT_TIMERS	(   TSS_LOG_EVENT_CONTROL_TIMERS	|\
				    TSS_LOG_EVENT_CONTROL_ANYRESULT	)

#define	LOG_EVENT_FAILURES	(   TSS_LOG_EVENT_CONTROL_FAIL		|\
				    TSS_LOG_EVENT_CONTROL_STOPPED	|\
				    TSS_LOG_EVENT_CONTROL_UNEVALUATED	|\
				    TSS_LOG_EVENT_CONTROL_TIMERS	|\
				    TSS_LOG_EVENT_CONTROL_COMMANDS	)

#define	LOG_EVENT_COMMANDS	(   LOG_EVENT_TIMERS	|\
				    TSS_LOG_EVENT_CONTROL_COMMANDS	)

#define	LOG_EVENT_ALL		(   LOG_EVENT_COMMANDS	|\
				    TSS_LOG_EVENT_CONTROL_ENVIRON	)
		    

/*
 * LogData_control values
 */
#define	TSS_LOG_DATA_CONTROL_NONE		0x00000001
#define	TSS_LOG_DATA_CONTROL_PASS		0x00000002
#define	TSS_LOG_DATA_CONTROL_FAIL		0x00000004
#define	TSS_LOG_DATA_CONTROL_WARNING		0x00000008
#define	TSS_LOG_DATA_CONTROL_STOPPED		0x00000010
#define	TSS_LOG_DATA_CONTROL_INFORMATIONAL	0x00000020
#define	TSS_LOG_DATA_CONTROL_COMPLETED		0x00000040
#define	TSS_LOG_DATA_CONTROL_UNEVALUATED	0x00000080

#define	TSS_LOG_DATA_CONTROL_ANYRESULT		0x000000FF


#define	LOG_DATA_OFF		0x00000000

#define	LOG_DATA_FAILURES	(   TSS_LOG_DATA_CONTROL_FAIL		|\
				    TSS_LOG_DATA_CONTROL_STOPPED	|\
				    TSS_LOG_DATA_CONTROL_UNEVALUATED	)

#define	LOG_DATA_ALL		TSS_LOG_DATA_CONTROL_ANYRESULT




/*
 * TSSDisable values
 */
    /* all datapool functions */
#define	TSS_DISABLE_DATAPOOLS		0x00000001

    /* TSSTimerStart, TSSTimerStop */
#define TSS_DISABLE_TIMERS		0x00000002

    /* TSSCommandStart, TSSCommandEnd, TSSLogCommand */
#define TSS_DISABLE_COMMANDS		0x00000004

    /* TSSThink, TSSThinkTime */
#define TSS_DISABLE_THINK		0x00000008

    /* TSSDelay */
#define TSS_DISABLE_DELAY		0x00000020

    /* TSSSyncpoint */
#define TSS_DISABLE_SYNCPOINTS		0x00000040

    /* access to shared variables */
#define TSS_DISABLE_SHAREDVARS		0x00000080

    /* all monitoring functions */
#define TSS_DISABLE_MONITOR		0x00000100

    /* all logging functions (including TSSLogCommand) */
#define TSS_DISABLE_LOG			0x00000200

    /* verification points */
#define TSS_DISABLE_VPS			0x00000400

#define TSS_DISABLE_ALL			0xFFFFFFFF


#define TSS_DISABLE_BLOCKING		(   TSS_DISABLE_THINK		|\
					    TSS_DISABLE_DELAY		|\
					    TSS_DISABLE_SYNCPOINTS	|\
					    TSS_DISABLE_SHAREDVARS	)

#define TSS_DISABLE_MEASUREMENT		(   TSS_DISABLE_TIMERS		|\
					    TSS_DISABLE_COMMANDS	|\
					    TSS_DISABLE_THINK		)

#define TSS_DISABLE_SYNCHONIZATION	(   TSS_DISABLE_SYNCPOINTS	|\
					    TSS_DISABLE_SHAREDVARS	)




/*
 * timer flags (boolean)
 */
#define TSS_TIMER_KEEP	0x00
#define TSS_TIMER_REMOVE	0x01


/*
 * TSSGetPath values
 */
#define	TSS_SOURCE_PATH			1
#define	TSS_ATTACHED_LOG_FILE_PATH	2




enum EvarOp
{
    EVOP_eval,
    EVOP_pop,
    EVOP_push,
    EVOP_reset,
    EVOP_restore,
    EVOP_save,
    EVOP_set,
    EVOP_END
};
typedef enum EvarOp EvarOp;


enum EvarKey
{
    EVAR_Think_avg = 0,
    EVAR_Think_sd,
    EVAR_Think_dist,
    EVAR_Think_def,
    EVAR_Think_max,

    EVAR_Think_dly_scale,
    EVAR_Think_cpu_threshold,
    EVAR_Think_cpu_dly_scale,
    EVAR_Initial_dly_max,
    EVAR_Delay_dly_scale,

    EVAR_Log_level,
    EVAR_Record_level,
    EVAR_Suspend_check,
    EVAR_LogEvent_control,
    EVAR_LogData_control,

    EVAR_TSSDisable,
    EVAR_END
};
typedef enum EvarKey EvarKey;



union EvarValue
{
    s32	envInt;
    char	*envStr;
};
typedef union EvarValue EvarValue;




enum IVKey
{
    IV_fcs_ts = 0,
    IV_lcs_ts,
    IV_fcr_ts,
    IV_lcr_ts,
    IV_lineno,
    IV_cmdcnt,
    IV_uid,
    IV_ncxmit,
    IV_ncrecv,
    IV_ncnull,
    IV_nusers,
    IV_nkxmit,
    IV_nrows,
    IV_ncols,
    IV_row,
    IV_col,
    IV_fs_ts,
    IV_ls_ts,
    IV_fr_ts,
    IV_lr_ts,
    IV_nxmit,
    IV_nrecv,
    IV_button_no,
    IV_fuxe_ts,
    IV_luxe_ts,
    IV_uxe_cnt,
    IV_ig_fs_ts,
    IV_ig_ls_ts,
    IV_ig_eot_ts,
    IV_prev_ig_fs_ts,
    IV_prev_ig_ls_ts,
    IV_npixels_act,
    IV_npixels_exp,
    IV_npixels_diff,
    IV_xwin_diff_level,
    IV_screen,
    IV_error,
    IV_total_rows,
    IV_statement_id,
    IV_error_logs,
    IV_cursor_id,
    IV_fc_ts,
    IV_lc_ts,
    IV_total_nrecv,
    IV_error_type,
    IV_tux_tpurcode,
    IV_command,
    IV_response,
    IV_source_file,
    IV_task_file,
    IV_cmd_id,
    IV_mcommand,
    IV_alltext,
    IV_error_text,
    IV_column_headers,
    IV_total_response,
    IV_script,
    IV_version,
    IV_user_group,
    IV_host,
    IV_refURI,
    IV_END
};
typedef enum IVKey IVKey;

union IVValue
{
    s32	ivInt;
    char	*ivStr;
};
typedef union IVValue IVValue;


/* 
 * shvaradj
 */
enum ShVarAdj
{
    SHVADJ_none = 0,
    SHVADJ_pre_inc = 'P',
    SHVADJ_post_inc = 'p',
    SHVADJ_pre_dec = 'M',
    SHVADJ_post_dec = 'm'
};
typedef enum ShVarAdj ShVarAdj;


/* 
 * shvarops
 */
enum ShVarOp
{
    SHVOP_assign = '=',
    SHVOP_add = '+',
    SHVOP_subtract = '-',
    SHVOP_multiply = '*',
    SHVOP_divide = '/',
    SHVOP_modulo = '%',
    SHVOP_and = '&',
    SHVOP_or = '|',
    SHVOP_xor = '^',
    SHVOP_shiftleft = '<',
    SHVOP_shiftright = '>'
};
typedef enum ShVarOp ShVarOp;


/*
 * named value structure
 */
typedef struct NamedValue NamedValue;
struct NamedValue
{
    char *name;
    char *value;
};

typedef struct TestCaseConfigurationAttribute TestCaseConfigurationAttribute;
struct TestCaseConfigurationAttribute
{
	char *name;
	char *op;
	char *value;
};

/*
 * application start/wait handle
 */
typedef void *TSSAppHandle;

/*
 * function prototypes
 */

#ifdef __cplusplus
extern "C" {
#endif

/*
 * hidden functions
 */

/*
 * advanced group
 */
s32 TSSThinkTime(s32 thinkAverage);
s32 TSSLogCommand(char *name, char *label, s16 result, char *description,
		s32 starttime, s32 endtime, char *logdata,
		s32 propertyCount, NamedValue *property);
s32 TSSInternalVarSet(IVKey internVar, IVValue internVal);

/*
 * session group
 */
s32 TSSServerStart(u16 *port);
s32 TSSServerStop(u16 port);
s32 TSSConnect(char *addr, u16 port, s32 id);
void TSSDisconnect(void);
s32 TSSContext(ContextKey ctx, void *value);
s32 TSSShutdown(void);

/* 
 * utility group
 */
s32 TSSErrorDetail(char *errorText, s32 *len);
char *TSSGetScriptOption(char *optionName);
char *TSSGetTestCaseName(void);
char *TSSGetTestCaseConfigurationName(void);
s32 TSSDelay(s32 msecs);
s32 TSSSeedRand(u32 seed);
s32 TSSRand(void);
s32 TSSUniform(s32 low, s32 high);
s32 TSSNegExp(s32 mean);
s32 TSSPrint(char *message);
s32 TSSePrint(char *message);
char *TSSUniqueString(void);
char *TSSGetPath(u32 pathKey);
TSSAppHandle TSSApplicationStart(char *application, char *workingDir,
			u32 flags);
TSSAppHandle TSSJavaApplicationStart(char *application, char *workingDir,
			char *ClassPath, char *JVM, char *JVMOptions);
s32 TSSApplicationWait(TSSAppHandle app, s32 *exitStatus, s32 timeout);
s32 TSSApplicationPid(TSSAppHandle app);
s32 TSSGetTestCaseConfigurationAttributeList(
			TestCaseConfigurationAttribute **config, s32 *count);
TestCaseConfigurationAttribute *
TSSGetTestCaseConfigurationAttribute(char *name);
s32 TSSGetComputerConfigurationAttributeList(NamedValue **config, s32 *count);
char *TSSGetComputerConfigurationAttributeValue(char *name);
char *TSSGetTestToolOption(char *optionName);
s32 TSSDownload(char **filenameList, s32 count);

/*
 * monitor group
 */
s32 TSSDisplay(char *message);
s32 TSSRunStateSet(RunState state);
s32 TSSRunStateGet(void);
s32 TSSPositionSet(char *srcFile, u32 lineNumber);
s32 TSSPositionGet(char **srcFile, u32 *lineNumber);
s32 TSSReportCommandStatus(s32 status);

/*
 * measurement group
 */
s32 TSSTimerStart(char *label, s32 timeStamp);
s32 TSSTimerStop(char *label, s32 timeStamp, u32 rmFlag);
s32 TSSCommandStart(char *label, char *name, RunState state);
s32 TSSCommandEnd(s16 result, char *description, s32 starttime, s32 endtime,
		char *logdata, s32 propertyCount, NamedValue *property);
s32 TSSThink(s32 thinkAverage);
s32 TSSGetTime(void);
s32 TSSEnvironmentOp(EvarKey envVar, EvarOp envOp, EvarValue *envVal);
s32 TSSInternalVarGet(IVKey internVar, IVValue *internVal);

/*
 * datapool group
 */
s32 TSSDatapoolOpen(char *name, u32 accessFlags, s32 overridecount, NamedValue *overrides);
s32 TSSDatapoolClose(s32 dpId);
s32 TSSDatapoolFetch(s32 dpId);
char *TSSDatapoolValue(s32 dpId, char *columnName);
s32 TSSDatapoolRewind(s32 dpId);
s32 TSSDatapoolSeek(s32 dpId, s32 count);
s32 TSSDatapoolSearch(s32 dpId, s32 count, NamedValue *keys);
s32 TSSDatapoolColumnCount(s32 dpId);
s32 TSSDatapoolRowCount(s32 dpId);
char *TSSDatapoolColumnName(s32 dpId, s32 column);

/*
 * synchronization group
 */
s32 TSSSyncPoint(char *label);
s32 TSSSharedVarEval(char *name, s32 *value, ShVarAdj op);
s32 TSSSharedVarAssign(char *name, s32 value, ShVarOp op, s32 *returnValue);
s32 TSSSharedVarWait(char *name, s32 min, s32 max, s32 adjust, s32 timeOut,
		 s32 *returnValue);

/*
 * logging group
 */
s32 TSSLogEvent(char *eventType, s16 result, char *description,
	    s32 propertyCount, NamedValue *property);
s32 TSSLogMessage(char *message, s16 result, char *description);
s32 TSSLogTestCaseResult(char *testcase, s16 result, char *description,
	    s32 propertyCount, NamedValue *property);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _RTTSS_H */

